@@ -1,5 +1,5 @@
 /* Definition for thread-local data handling.  linuxthreads/i386 version.
-   Copyright (C) 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -24,6 +24,7 @@
 # include <pt-machine.h>
 
 #ifndef __ASSEMBLER__
+# include <stdbool.h>
 # include <stddef.h>
 # include <stdint.h>
 
@@ -31,7 +32,11 @@
 typedef union dtv
 {
   size_t counter;
-  void *pointer;
+  struct
+  {
+    void *val;
+    bool is_static;
+  } pointer;
 } dtv_t;
 
 
@@ -42,9 +47,9 @@ typedef struct
   dtv_t *dtv;
   void *self;		/* Pointer to the thread descriptor.  */
   int multiple_threads;
-#ifdef NEED_DL_SYSINFO
   uintptr_t sysinfo;
-#endif
+  uintptr_t stack_guard;
+  uintptr_t pointer_guard;
 } tcbhead_t;
 
 #else /* __ASSEMBLER__ */
@@ -113,12 +118,14 @@ typedef struct
 #   define TLS_LOAD_EBX
 #  endif
 
+#  if !defined IS_IN_linuxthreads && !defined DO_MODIFY_LDT
+#   include "useldt.h"		/* For the structure.  */
+#  endif
 #  if __ASSUME_LDT_WORKS > 0
 #   define TLS_DO_MODIFY_LDT_KERNEL_CHECK(doit) (doit) /* Nothing to check.  */
 #  else
-#   include "useldt.h"		/* For the structure.  */
 #   define TLS_DO_MODIFY_LDT_KERNEL_CHECK(doit)				      \
-  (__builtin_expect (GL(dl_osversion) < 131939, 0)			      \
+  (__builtin_expect (GLRO(dl_osversion) < 131939, 0)			      \
    ? "kernel too old for thread-local storage support\n"			      \
    : (doit))
 #  endif
@@ -184,7 +191,7 @@ TLS_DO_MODIFY_LDT_KERNEL_CHECK(						      \
 
 #if defined NEED_DL_SYSINFO
 # define INIT_SYSINFO \
-  head->sysinfo = GL(dl_sysinfo)
+  head->sysinfo = GLRO(dl_sysinfo)
 #else
 # define INIT_SYSINFO
 #endif
@@ -217,6 +224,23 @@ TLS_DO_MODIFY_LDT_KERNEL_CHECK(						      \
   ({ struct _pthread_descr_struct *__descr;				      \
      THREAD_GETMEM (__descr, p_header.data.dtvp); })
 
+
+/* Set the stack guard field in TCB head.  */
+#define THREAD_SET_STACK_GUARD(value) \
+  THREAD_SETMEM (THREAD_SELF, p_header.data.stack_guard, value)
+#define THREAD_COPY_STACK_GUARD(descr) \
+  ((descr)->p_header.data.stack_guard				      \
+   = THREAD_GETMEM (THREAD_SELF, p_header.data.stack_guard))
+
+
+/* Set the pointer guard field in the TCB head.  */
+#define THREAD_SET_POINTER_GUARD(value) \
+  THREAD_SETMEM (THREAD_SELF, p_header.data.pointer_guard, value)
+#define THREAD_COPY_POINTER_GUARD(descr) \
+  ((descr)->p_header.data.pointer_guard				      \
+   = THREAD_GETMEM (THREAD_SELF, p_header.data.pointer_guard))
+
+
 # endif	/* HAVE_TLS_SUPPORT && (FLOATING_STACKS || !IS_IN_libpthread) */
 #endif /* __ASSEMBLER__ */
 
