@@ -1,6 +1,6 @@
 /* Machine-dependent pthreads configuration and inline functions.
    IA-64 version.
-   Copyright (C) 1999, 2000, 2002, 2003 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2002, 2003, 2004 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
 #include <ia64intrin.h>
 
 #ifndef PT_EI
-# define PT_EI extern inline
+# define PT_EI extern inline __attribute__ ((always_inline))
 #endif
 
 extern long int testandset (int *spinlock);
@@ -89,7 +89,7 @@ __compare_and_swap (long int *p, long int oldval, long int newval)
        ("mov ar.ccv=%4;;\n\t"
 	"cmpxchg8.acq %0=%1,%2,ar.ccv"
 	: "=r" (readval), "=m" (__atomic_fool_gcc (p))
-	: "r"(newval), "1" (__atomic_fool_gcc (p)), "r" (oldval)
+	: "r"(newval), "m" (__atomic_fool_gcc (p)), "r" (oldval)
 	: "memory");
   return readval == oldval;
 }
@@ -105,7 +105,7 @@ __compare_and_swap_with_release_semantics (long int *p,
        ("mov ar.ccv=%4;;\n\t"
 	"cmpxchg8.rel %0=%1,%2,ar.ccv"
 	: "=r" (readval), "=m" (__atomic_fool_gcc (p))
-	: "r"(newval), "1" (__atomic_fool_gcc (p)), "r" (oldval)
+	: "r"(newval), "m" (__atomic_fool_gcc (p)), "r" (oldval)
 	: "memory");
   return readval == oldval;
 }
@@ -121,10 +121,13 @@ testandset (int *spinlock)
   __asm__ __volatile__(
        "xchg4 %0=%1,%2"
        : "=r"(ret), "=m"(__atomic_fool_gcc (spinlock))
-       : "r"(1), "1"(__atomic_fool_gcc (spinlock))
+       : "r"(1), "m"(__atomic_fool_gcc (spinlock))
        : "memory");
 
   return ret;
 }
 
+/* Indicate that we are looping.  */
+#define BUSY_WAIT_NOP	__asm__ ("hint @pause")
+
 #endif /* pt-machine.h */
